@@ -2,12 +2,14 @@
 This module interfaces with PROJ to produce a pythonic interface
 to the coordinate reference system (CRS) information.
 """
+
 # pylint: disable=too-many-lines
 import json
 import re
 import threading
 import warnings
-from typing import Any, Callable, Dict, List, Optional, Tuple, Union
+from collections.abc import Callable
+from typing import Any, Optional
 
 from pyproj._crs import (
     _CRS,
@@ -37,6 +39,17 @@ from pyproj.enums import ProjVersion, WktVersion
 from pyproj.exceptions import CRSError
 from pyproj.geod import Geod
 
+_RE_PROJ_PARAM = re.compile(
+    r"""
+    \+              # parameter starts with '+' character
+    (?P<param>\w+)    # capture parameter name
+    \=?             # match both key only and key-value parameters
+    (?P<value>\S+)? # capture all characters up to next space (None if no value)
+    \s*?            # consume remaining whitespace, if any
+""",
+    re.X,
+)
+
 
 class CRSLocal(threading.local):
     """
@@ -78,7 +91,7 @@ def _prepare_from_proj_string(in_crs_string: str) -> str:
     # make sure the projection starts with +proj or +init
     starting_params = ("+init", "+proj", "init", "proj")
     if not in_crs_string.startswith(starting_params):
-        kvpairs: List[str] = []
+        kvpairs: list[str] = []
         first_item_inserted = False
         for kvpair in in_crs_string.split():
             if not first_item_inserted and (kvpair.startswith(starting_params)):
@@ -131,12 +144,12 @@ def _prepare_from_string(in_crs_string: str) -> str:
     return in_crs_string
 
 
-def _prepare_from_authority(auth_name: str, auth_code: Union[str, int]):
+def _prepare_from_authority(auth_name: str, auth_code: str | int):
     return f"{auth_name}:{auth_code}"
 
 
-def _prepare_from_epsg(auth_code: Union[str, int]):
-    return _prepare_from_authority("epsg", auth_code)
+def _prepare_from_epsg(auth_code: str | int):
+    return _prepare_from_authority("EPSG", auth_code)
 
 
 def _is_epsg_code(auth_code: Any) -> bool:
@@ -155,19 +168,21 @@ class CRS:
 
     .. versionadded:: 2.0.0
 
+    See: :c:func:`proj_create`
+
     The functionality is based on other fantastic projects:
 
-    * `rasterio <https://github.com/mapbox/rasterio/blob/c13f0943b95c0eaa36ff3f620bd91107aa67b381/rasterio/_crs.pyx>`_  # noqa: E501
-    * `opendatacube <https://github.com/opendatacube/datacube-core/blob/83bae20d2a2469a6417097168fd4ede37fd2abe5/datacube/utils/geometry/_base.py>`_  # noqa: E501
+    * `rasterio <https://github.com/mapbox/rasterio/blob/c13f0943b95c0eaa36ff3f620bd91107aa67b381/rasterio/_crs.pyx>`_
+    * `opendatacube <https://github.com/opendatacube/datacube-core/blob/83bae20d2a2469a6417097168fd4ede37fd2abe5/datacube/utils/geometry/_base.py>`_
 
     Attributes
     ----------
     srs: str
         The string form of the user input used to create the CRS.
 
-    """
+    """  # noqa: E501
 
-    def __init__(self, projparams: Any = None, **kwargs) -> None:
+    def __init__(self, projparams: Any | None = None, **kwargs) -> None:
         """
         Initialize a CRS class instance with:
           - PROJ string
@@ -319,7 +334,7 @@ class CRS:
             elif isinstance(projparams, (list, tuple)) and len(projparams) == 2:
                 projstring = _prepare_from_authority(*projparams)
             elif hasattr(projparams, "to_wkt"):
-                projstring = projparams.to_wkt()  # type: ignore
+                projstring = projparams.to_wkt()
             else:
                 raise CRSError(f"Invalid CRS input: {projparams!r}")
 
@@ -344,7 +359,7 @@ class CRS:
         return self._local.crs
 
     @classmethod
-    def from_authority(cls, auth_name: str, code: Union[str, int]) -> "CRS":
+    def from_authority(cls, auth_name: str, code: str | int) -> "CRS":
         """
         .. versionadded:: 2.2.0
 
@@ -364,7 +379,7 @@ class CRS:
         return cls.from_user_input(_prepare_from_authority(auth_name, code))
 
     @classmethod
-    def from_epsg(cls, code: Union[str, int]) -> "CRS":
+    def from_epsg(cls, code: str | int) -> "CRS":
         """Make a CRS from an EPSG code
 
         Parameters
@@ -487,7 +502,7 @@ class CRS:
             return value
         return cls(value, **kwargs)
 
-    def get_geod(self) -> Optional[Geod]:
+    def get_geod(self) -> Geod | None:
         """
         Returns
         -------
@@ -564,16 +579,20 @@ class CRS:
 
         .. warning:: You will likely lose important projection
           information when converting to a PROJ string from
-          another format. See: https://proj.org/faq.html#what-is-the-best-format-for-describing-coordinate-reference-systems  # noqa: E501
+          another format. See: https://proj.org/faq.html#what-is-the-best-format-for-describing-coordinate-reference-systems
 
         Returns
         -------
         dict:
             PROJ params in dict format.
 
-        """
+        """  # noqa: E501
 
-        def parse(val):
+        proj_string = self.to_proj4()
+        if proj_string is None:
+            return {}
+
+        def _parse(val):
             if val.lower() == "true":
                 return True
             if val.lower() == "false":
@@ -588,20 +607,19 @@ class CRS:
                 pass
             return _try_list_if_string(val)
 
-        proj_string = self.to_proj4()
-        if proj_string is None:
-            return {}
+        proj_dict = {}
+        for param in _RE_PROJ_PARAM.finditer(proj_string):
+            key, value = param.groups()
+            if value is not None:
+                value = _parse(value)
+            if value is not False:
+                proj_dict[key] = value
 
-        items = map(
-            lambda kv: len(kv) == 2 and (kv[0], parse(kv[1])) or (kv[0], None),
-            (part.lstrip("+").split("=", 1) for part in proj_string.strip().split()),
-        )
-
-        return {key: value for key, value in items if value is not False}
+        return proj_dict
 
     def to_cf(
         self,
-        wkt_version: Union[WktVersion, str] = WktVersion.WKT2_2019,
+        wkt_version: WktVersion | str = WktVersion.WKT2_2019,
         errcheck: bool = False,
     ) -> dict:
         """
@@ -626,8 +644,8 @@ class CRS:
             CF-1.8 version of the projection.
 
         """
-        # pylint: disable=too-many-branches
-        cf_dict: Dict[str, Any] = {"crs_wkt": self.to_wkt(wkt_version)}
+        # pylint: disable=too-many-branches,too-many-return-statements
+        cf_dict: dict[str, Any] = {"crs_wkt": self.to_wkt(wkt_version)}
 
         # handle bound CRS
         if (
@@ -636,7 +654,7 @@ class CRS:
             and self.coordinate_operation.towgs84
             and self.source_crs
         ):
-            sub_cf: Dict[str, Any] = self.source_crs.to_cf(
+            sub_cf: dict[str, Any] = self.source_crs.to_cf(
                 wkt_version=wkt_version,
                 errcheck=errcheck,
             )
@@ -677,23 +695,31 @@ class CRS:
         # handle geographic CRS
         if self.geodetic_crs:
             cf_dict["geographic_crs_name"] = self.geodetic_crs.name
+            if self.geodetic_crs.datum:
+                cf_dict["horizontal_datum_name"] = self.geodetic_crs.datum.name
 
         if self.is_geographic:
             if self.coordinate_operation:
+                if (
+                    self.coordinate_operation.method_name.lower()
+                    not in _INVERSE_GEOGRAPHIC_GRID_MAPPING_NAME_MAP
+                ):
+                    if errcheck:
+                        warnings.warn(
+                            "Unsupported coordinate operation: "
+                            f"{self.coordinate_operation.method_name}"
+                        )
+                    return {"crs_wkt": cf_dict["crs_wkt"]}
                 cf_dict.update(
                     _INVERSE_GEOGRAPHIC_GRID_MAPPING_NAME_MAP[
                         self.coordinate_operation.method_name.lower()
                     ](self.coordinate_operation)
                 )
-                if self.datum:
-                    cf_dict["horizontal_datum_name"] = self.datum.name
             else:
                 cf_dict["grid_mapping_name"] = "latitude_longitude"
             return cf_dict
 
         # handle projected CRS
-        if self.is_projected and self.datum:
-            cf_dict["horizontal_datum_name"] = self.datum.name
         coordinate_operation = None
         if not self.is_bound and self.is_projected:
             coordinate_operation = self.coordinate_operation
@@ -713,7 +739,7 @@ class CRS:
                 else:
                     warnings.warn("Coordinate operation not found.")
 
-            return {"crs_wkt": self.to_wkt(wkt_version)}
+            return {"crs_wkt": cf_dict["crs_wkt"]}
 
         cf_dict.update(
             _INVERSE_GRID_MAPPING_NAME_MAP[coordinate_operation_name](
@@ -725,18 +751,15 @@ class CRS:
     @staticmethod
     def from_cf(
         in_cf: dict,
-        ellipsoidal_cs: Any = None,
-        cartesian_cs: Any = None,
-        vertical_cs: Any = None,
-        errcheck=False,
+        ellipsoidal_cs: Any | None = None,
+        cartesian_cs: Any | None = None,
+        vertical_cs: Any | None = None,
     ) -> "CRS":
         """
         .. versionadded:: 2.2.0
 
         .. versionadded:: 3.0.0 ellipsoidal_cs, cartesian_cs, vertical_cs
 
-        .. deprecated:: 3.2.0 errcheck
-
         This converts a Climate and Forecast (CF) Grid Mapping Version 1.8
         dict to a :obj:`pyproj.crs.CRS` object.
 
@@ -758,21 +781,12 @@ class CRS:
             Input to create a Vertical Coordinate System accepted by
             :meth:`pyproj.crs.CoordinateSystem.from_user_input`
             or :class:`pyproj.crs.coordinate_system.VerticalCS`
-        errcheck: bool, default=False
-            This parameter is for backwards compatibility with the old version.
-            It currently does nothing when True or False. DEPRECATED.
 
         Returns
         -------
         CRS
         """
         # pylint: disable=too-many-branches
-        if errcheck:
-            warnings.warn(
-                "errcheck is deprecated as it does nothing.",
-                DeprecationWarning,
-                stacklevel=2,
-            )
 
         unknown_names = ("unknown", "undefined")
         if "crs_wkt" in in_cf:
@@ -789,9 +803,9 @@ class CRS:
 
         # build geographic CRS
         try:
-            geographic_conversion_method: Optional[
-                Callable
-            ] = _GEOGRAPHIC_GRID_MAPPING_NAME_MAP[grid_mapping_name]
+            geographic_conversion_method: None | Callable = (
+                _GEOGRAPHIC_GRID_MAPPING_NAME_MAP[grid_mapping_name]
+            )
         except KeyError:
             geographic_conversion_method = None
 
@@ -806,9 +820,9 @@ class CRS:
             geographic_crs = CRS(geographic_crs_name)
             if ellipsoidal_cs is not None:
                 geographic_crs_json = geographic_crs.to_json_dict()
-                geographic_crs_json[
-                    "coordinate_system"
-                ] = CoordinateSystem.from_user_input(ellipsoidal_cs).to_json_dict()
+                geographic_crs_json["coordinate_system"] = (
+                    CoordinateSystem.from_user_input(ellipsoidal_cs).to_json_dict()
+                )
                 geographic_crs = CRS(geographic_crs_json)
         else:
             geographic_crs = GeographicCRS(ellipsoidal_cs=ellipsoidal_cs)
@@ -861,7 +875,7 @@ class CRS:
             name="undefined", components=[bound_crs or projected_crs, vertical_crs]
         )
 
-    def cs_to_cf(self) -> List[dict]:
+    def cs_to_cf(self) -> list[dict]:
         """
         .. versionadded:: 3.0.0
 
@@ -872,7 +886,7 @@ class CRS:
 
         Returns
         -------
-        List[dict]:
+        list[dict]:
             CF-1.8 version of the coordinate systems.
         """
         cf_axis_list = []
@@ -1021,18 +1035,18 @@ class CRS:
         )
 
     @property
-    def sub_crs_list(self) -> List["CRS"]:
+    def sub_crs_list(self) -> list["CRS"]:
         """
         If the CRS is a compound CRS, it will return a list of sub CRS objects.
 
         Returns
         -------
-        List[CRS]
+        list[CRS]
         """
         return [self.__class__(sub_crs) for sub_crs in self._crs.sub_crs_list]
 
     @property
-    def utm_zone(self) -> Optional[str]:
+    def utm_zone(self) -> str | None:
         """
         .. versionadded:: 2.6.0
 
@@ -1040,7 +1054,7 @@ class CRS:
 
         Returns
         -------
-        Optional[str]:
+        str | None:
             The UTM zone number and letter if applicable.
         """
         if self.is_bound and self.source_crs:
@@ -1077,7 +1091,7 @@ class CRS:
         return self._crs.type_name
 
     @property
-    def axis_info(self) -> List[Axis]:
+    def axis_info(self) -> list[Axis]:
         """
         Retrieves all relevant axis information in the CRS.
         If it is a Bound CRS, it gets the axis list from the Source CRS.
@@ -1085,13 +1099,13 @@ class CRS:
 
         Returns
         -------
-        List[Axis]:
+        list[Axis]:
             The list of axis information.
         """
         return self._crs.axis_info
 
     @property
-    def area_of_use(self) -> Optional[AreaOfUse]:
+    def area_of_use(self) -> AreaOfUse | None:
         """
         Returns
         -------
@@ -1101,7 +1115,7 @@ class CRS:
         return self._crs.area_of_use
 
     @property
-    def ellipsoid(self) -> Optional[Ellipsoid]:
+    def ellipsoid(self) -> Ellipsoid | None:
         """
         .. versionadded:: 2.2.0
 
@@ -1113,7 +1127,7 @@ class CRS:
         return self._crs.ellipsoid
 
     @property
-    def prime_meridian(self) -> Optional[PrimeMeridian]:
+    def prime_meridian(self) -> PrimeMeridian | None:
         """
         .. versionadded:: 2.2.0
 
@@ -1125,7 +1139,7 @@ class CRS:
         return self._crs.prime_meridian
 
     @property
-    def datum(self) -> Optional[Datum]:
+    def datum(self) -> Datum | None:
         """
         .. versionadded:: 2.2.0
 
@@ -1136,7 +1150,7 @@ class CRS:
         return self._crs.datum
 
     @property
-    def coordinate_system(self) -> Optional[CoordinateSystem]:
+    def coordinate_system(self) -> CoordinateSystem | None:
         """
         .. versionadded:: 2.2.0
 
@@ -1147,7 +1161,7 @@ class CRS:
         return self._crs.coordinate_system
 
     @property
-    def coordinate_operation(self) -> Optional[CoordinateOperation]:
+    def coordinate_operation(self) -> CoordinateOperation | None:
         """
         .. versionadded:: 2.2.0
 
@@ -1183,8 +1197,9 @@ class CRS:
 
     def to_wkt(
         self,
-        version: Union[WktVersion, str] = WktVersion.WKT2_2019,
+        version: WktVersion | str = WktVersion.WKT2_2019,
         pretty: bool = False,
+        output_axis_rule: bool | None = None,
     ) -> str:
         """
         Convert the projection to a WKT string.
@@ -1197,6 +1212,7 @@ class CRS:
           - WKT1_GDAL
           - WKT1_ESRI
 
+        .. versionadded:: 3.6.0 output_axis_rule
 
         Parameters
         ----------
@@ -1205,19 +1221,21 @@ class CRS:
             Default is :attr:`pyproj.enums.WktVersion.WKT2_2019`.
         pretty: bool, default=False
             If True, it will set the output to be a multiline string.
+        output_axis_rule: bool, optional, default=None
+            If True, it will set the axis rule on any case. If false, never.
+            None for AUTO, that depends on the CRS and version.
 
         Returns
         -------
         str
         """
-        wkt = self._crs.to_wkt(version=version, pretty=pretty)
+        wkt = self._crs.to_wkt(
+            version=version, pretty=pretty, output_axis_rule=output_axis_rule
+        )
         if wkt is None:
-            warnings.warn(
+            raise CRSError(
                 f"CRS cannot be converted to a WKT string of a '{version}' version. "
-                "Select a different version of a WKT string or edit your CRS. Future"
-                "versions of pyproj will raise a CRSError in this case.",
-                FutureWarning,
-                stacklevel=2,
+                "Select a different version of a WKT string or edit your CRS."
             )
         return wkt
 
@@ -1240,12 +1258,7 @@ class CRS:
         """
         proj_json = self._crs.to_json(pretty=pretty, indentation=indentation)
         if proj_json is None:
-            warnings.warn(
-                "CRS cannot be converted to a PROJ JSON string. "
-                "Future versions of pyproj will raise a CRSError in this case.",
-                FutureWarning,
-                stacklevel=2,
-            )
+            raise CRSError("CRS cannot be converted to a PROJ JSON string.")
         return proj_json
 
     def to_json_dict(self) -> dict:
@@ -1260,14 +1273,14 @@ class CRS:
         """
         return self._crs.to_json_dict()
 
-    def to_proj4(self, version: Union[ProjVersion, int] = ProjVersion.PROJ_5) -> str:
+    def to_proj4(self, version: ProjVersion | int = ProjVersion.PROJ_5) -> str:
         """
         Convert the projection to a PROJ string.
 
         .. warning:: You will likely lose important projection
           information when converting to a PROJ string from
           another format. See:
-          https://proj.org/faq.html#what-is-the-best-format-for-describing-coordinate-reference-systems  # noqa: E501
+          https://proj.org/faq.html#what-is-the-best-format-for-describing-coordinate-reference-systems
 
         Parameters
         ----------
@@ -1278,18 +1291,13 @@ class CRS:
         Returns
         -------
         str
-        """
+        """  # noqa: E501
         proj = self._crs.to_proj4(version=version)
         if proj is None:
-            warnings.warn(
-                "CRS cannot be converted to a PROJ string. "
-                "Future versions of pyproj will raise a CRSError in this case.",
-                FutureWarning,
-                stacklevel=2,
-            )
+            raise CRSError("CRS cannot be converted to a PROJ string.")
         return proj
 
-    def to_epsg(self, min_confidence: int = 70) -> Optional[int]:
+    def to_epsg(self, min_confidence: int = 70) -> int | None:
         """
         Return the EPSG code best matching the CRS
         or None if it a match is not found.
@@ -1297,7 +1305,7 @@ class CRS:
         Example:
 
         >>> from pyproj import CRS
-        >>> ccs = CRS("epsg:4328")
+        >>> ccs = CRS("EPSG:4328")
         >>> ccs.to_epsg()
         4328
 
@@ -1321,12 +1329,12 @@ class CRS:
 
         Returns
         -------
-        Optional[int]:
+        int | None:
             The best matching EPSG code matching the confidence level.
         """
         return self._crs.to_epsg(min_confidence=min_confidence)
 
-    def to_authority(self, auth_name: Optional[str] = None, min_confidence: int = 70):
+    def to_authority(self, auth_name: str | None = None, min_confidence: int = 70):
         """
         .. versionadded:: 2.2.0
 
@@ -1336,7 +1344,7 @@ class CRS:
         Example:
 
         >>> from pyproj import CRS
-        >>> ccs = CRS("epsg:4328")
+        >>> ccs = CRS("EPSG:4328")
         >>> ccs.to_authority()
         ('EPSG', '4328')
 
@@ -1369,8 +1377,8 @@ class CRS:
         )
 
     def list_authority(
-        self, auth_name: Optional[str] = None, min_confidence: int = 70
-    ) -> List[AuthorityMatchInfo]:
+        self, auth_name: str | None = None, min_confidence: int = 70
+    ) -> list[AuthorityMatchInfo]:
         """
         .. versionadded:: 3.2.0
 
@@ -1379,7 +1387,7 @@ class CRS:
         Example:
 
         >>> from pyproj import CRS
-        >>> ccs = CRS("epsg:4328")
+        >>> ccs = CRS("EPSG:4328")
         >>> ccs.list_authority()
         [AuthorityMatchInfo(auth_name='EPSG', code='4326', confidence=100)]
 
@@ -1405,14 +1413,14 @@ class CRS:
 
         Returns
         -------
-        List[AuthorityMatchInfo]:
+        list[AuthorityMatchInfo]:
             List of authority matches for the CRS.
         """
         return self._crs.list_authority(
             auth_name=auth_name, min_confidence=min_confidence
         )
 
-    def to_3d(self, name: Optional[str] = None) -> "CRS":
+    def to_3d(self, name: str | None = None) -> "CRS":
         """
         .. versionadded:: 3.1.0
 
@@ -1434,12 +1442,29 @@ class CRS:
         """
         return self.__class__(self._crs.to_3d(name=name))
 
+    def to_2d(self, name: str | None = None) -> "CRS":
+        """
+        .. versionadded:: 3.6.0
+
+        Convert the current CRS to the 2D version if it makes sense.
+
+        Parameters
+        ----------
+        name: str, optional
+            CRS name. Defaults to use the name of the original CRS.
+
+        Returns
+        -------
+        CRS
+        """
+        return self.__class__(self._crs.to_2d(name=name))
+
     @property
     def is_geographic(self) -> bool:
         """
         This checks if the CRS is geographic.
         It will check if it has a geographic CRS
-        in the sub CRS if it is a compount CRS and will check if
+        in the sub CRS if it is a compound CRS and will check if
         the source CRS is geographic if it is a bound CRS.
 
         Returns
@@ -1454,7 +1479,7 @@ class CRS:
         """
         This checks if the CRS is projected.
         It will check if it has a projected CRS
-        in the sub CRS if it is a compount CRS and will check if
+        in the sub CRS if it is a compound CRS and will check if
         the source CRS is projected if it is a bound CRS.
 
         Returns
@@ -1471,7 +1496,7 @@ class CRS:
 
         This checks if the CRS is vertical.
         It will check if it has a vertical CRS
-        in the sub CRS if it is a compount CRS and will check if
+        in the sub CRS if it is a compound CRS and will check if
         the source CRS is vertical if it is a bound CRS.
 
         Returns
@@ -1540,13 +1565,38 @@ class CRS:
         """
         return self._crs.is_derived
 
-    def __eq__(self, other: Any) -> bool:
+    @property
+    def is_deprecated(self) -> bool:
+        """
+        .. versionadded:: 3.7.0
+
+        Check if the CRS is deprecated
+
+        Returns
+        -------
+        bool
+        """
+        return self._crs.is_deprecated
+
+    def get_non_deprecated(self) -> list["CRS"]:
+        """
+        .. versionadded:: 3.7.0
+
+        Return a list of non-deprecated objects related to this.
+
+        Returns
+        -------
+        list[CRS]
+        """
+        return self._crs.get_non_deprecated()
+
+    def __eq__(self, other: object) -> bool:
         return self.equals(other)
 
-    def __getstate__(self) -> Dict[str, str]:
+    def __getstate__(self) -> dict[str, str]:
         return {"srs": self.srs}
 
-    def __setstate__(self, state: Dict[str, Any]):
+    def __setstate__(self, state: dict[str, Any]):
         self.__dict__.update(state)
         self._local = CRSLocal()
 
@@ -1558,7 +1608,7 @@ class CRS:
 
     def __repr__(self) -> str:
         # get axis information
-        axis_info_list: List[str] = []
+        axis_info_list: list[str] = []
         for axis in self.axis_info:
             axis_info_list.extend(["- ", str(axis), "\n"])
         axis_info_str = "".join(axis_info_list)
@@ -1625,7 +1675,7 @@ class CustomConstructorCRS(CRS):
     """
 
     @property
-    def _expected_types(self) -> Tuple[str, ...]:
+    def _expected_types(self) -> tuple[str, ...]:
         """
         These are the type names of the CRS class
         that are expected when using the from_* methods.
@@ -1712,17 +1762,17 @@ class CustomConstructorCRS(CRS):
         return None if self._crs.target_crs is None else CRS(self._crs.target_crs)
 
     @property
-    def sub_crs_list(self) -> List["CRS"]:
+    def sub_crs_list(self) -> list["CRS"]:
         """
         If the CRS is a compound CRS, it will return a list of sub CRS objects.
 
         Returns
         -------
-        List[CRS]
+        list[CRS]
         """
         return [CRS(sub_crs) for sub_crs in self._crs.sub_crs_list]
 
-    def to_3d(self, name: Optional[str] = None) -> "CRS":
+    def to_3d(self, name: str | None = None) -> "CRS":
         """
         .. versionadded:: 3.1.0
 
@@ -1757,15 +1807,15 @@ class GeographicCRS(CustomConstructorCRS):
     def __init__(
         self,
         name: str = "undefined",
-        datum: Any = "urn:ogc:def:datum:EPSG::6326",
-        ellipsoidal_cs: Any = None,
+        datum: Any = "urn:ogc:def:ensemble:EPSG::6326",
+        ellipsoidal_cs: Any | None = None,
     ) -> None:
         """
         Parameters
         ----------
         name: str, default="undefined"
             Name of the CRS.
-        datum: Any, default="urn:ogc:def:datum:EPSG::6326"
+        datum: Any, default="urn:ogc:def:ensemble:EPSG::6326"
             Anything accepted by :meth:`pyproj.crs.Datum.from_user_input` or
             a :class:`pyproj.crs.datum.CustomDatum`.
         ellipsoidal_cs: Any, optional
@@ -1773,15 +1823,19 @@ class GeographicCRS(CustomConstructorCRS):
             Anything accepted by :meth:`pyproj.crs.CoordinateSystem.from_user_input`
             or an Ellipsoidal Coordinate System created from :ref:`coordinate_system`.
         """
+        datum = Datum.from_user_input(datum).to_json_dict()
         geographic_crs_json = {
             "$schema": "https://proj.org/schemas/v0.2/projjson.schema.json",
             "type": "GeographicCRS",
             "name": name,
-            "datum": Datum.from_user_input(datum).to_json_dict(),
             "coordinate_system": CoordinateSystem.from_user_input(
                 ellipsoidal_cs or Ellipsoidal2DCS()
             ).to_json_dict(),
         }
+        if datum["type"] == "DatumEnsemble":
+            geographic_crs_json["datum_ensemble"] = datum
+        else:
+            geographic_crs_json["datum"] = datum
         super().__init__(geographic_crs_json)
 
 
@@ -1802,7 +1856,7 @@ class DerivedGeographicCRS(CustomConstructorCRS):
         self,
         base_crs: Any,
         conversion: Any,
-        ellipsoidal_cs: Any = None,
+        ellipsoidal_cs: Any | None = None,
         name: str = "undefined",
     ) -> None:
         """
@@ -1904,8 +1958,8 @@ class ProjectedCRS(CustomConstructorCRS):
         self,
         conversion: Any,
         name: str = "undefined",
-        cartesian_cs: Any = None,
-        geodetic_crs: Any = None,
+        cartesian_cs: Any | None = None,
+        geodetic_crs: Any | None = None,
     ) -> None:
         """
         Parameters
@@ -1944,7 +1998,7 @@ class VerticalCRS(CustomConstructorCRS):
     """
     .. versionadded:: 2.5.0
 
-    This class is for building a Vetical CRS.
+    This class is for building a Vertical CRS.
 
     .. warning:: geoid_model support only exists in PROJ >= 6.3.0
 
@@ -1956,8 +2010,8 @@ class VerticalCRS(CustomConstructorCRS):
         self,
         name: str,
         datum: Any,
-        vertical_cs: Any = None,
-        geoid_model: Optional[str] = None,
+        vertical_cs: Any | None = None,
+        geoid_model: str | None = None,
     ) -> None:
         """
         Parameters
@@ -1997,13 +2051,13 @@ class CompoundCRS(CustomConstructorCRS):
 
     _expected_types = ("Compound CRS",)
 
-    def __init__(self, name: str, components: List[Any]) -> None:
+    def __init__(self, name: str, components: list[Any]) -> None:
         """
         Parameters
         ----------
         name: str
             The name of the Compound CRS.
-        components: List[Any], optional
+        components: list[Any], optional
             List of CRS to create a Compound Coordinate System.
             List of anything accepted by :meth:`pyproj.crs.CRS.from_user_input`
         """
